/* This is a simple tape test program which writes to a tape blocks of random length
   and read them back.

   File tapetest.c  Nov 11, 1992 Kai Makisara
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

main()
{
    int i, fd;
    int *ibuf;
    int count, total;
    int min_transfer, max_transfer, transfer;
    unsigned int divisor;

    printf("Minimum and maximum transfer lengths? ");
    scanf("%d,%d", &min_transfer, &max_transfer);
    min_transfer &= 0x7ffffffc;  /* Transfer words */
    max_transfer &= 0x7ffffffc;
    if (min_transfer > max_transfer) {
      fprintf(stderr, "Wrong limits.\n");
      exit(1);
    }
    divisor = (32768U * 65536U - 1) / (max_transfer - min_transfer);

    if ((ibuf = (int *)malloc(max_transfer)) == NULL) {
      fprintf(stderr, "Can't allocate buffer.\n");
      exit(1);
    }

    printf("File length in megabytes? ");
    scanf("%d", &total);
    total *= 1024 * 1024;

    if ((fd = open("/dev/rmt0h", O_RDWR)) < 0) {
      fprintf(stderr, "Can't open tape.\n");
      exit(1);
    }

    for (count = 0; count < total; ) {
      transfer = min_transfer + (random() / divisor);
      transfer &= 0x7ffffffc;
/*      printf("%d\n", transfer, count); */
      if (transfer > max_transfer)
	transfer = max_transfer;
      for (i=0; i < transfer; i += 4)
	ibuf[i/4] = (count + i) / 4;
      if ((i = write(fd, (char *)ibuf, transfer)) != transfer) {
	fprintf(stderr, "Write error at %d.\n", i);
	exit(1);
      }
      count += transfer;
    }
    close(fd);
    total = count;
    printf("Number of written bytes is %d.\n", total);
    printf("Rewinding tape.\n");

    if ((fd = open("/dev/rmt0h", O_RDONLY)) < 0) {
      fprintf(stderr, "Can't open tape.\n");
      exit(1);
    }

    for (count = 0; count < total; ) {
      transfer = min_transfer + (random() / divisor);
      transfer &= 0x7ffffffc;
/*      printf("%d\n", transfer, count); */
      if (transfer > max_transfer)
	transfer = max_transfer;
      if ((i = read(fd, (char *)ibuf, transfer)) != transfer)
	if ((count + i) >= total)
	  printf("Correctly truncated last read.\n");
	else {
	  fprintf(stderr, "Read error at %d.\n", count + i);
	  exit(1);
	}
      if (count + transfer > total) {
	transfer = total - count;
	printf("Peek over the cliff: %d (should be 0).\n", ibuf[transfer/4]);
      }
      for (i=0; i < transfer; i += 4)
	if (ibuf[i/4] != (count + i) / 4) {
	  printf("Value error at %d (%d) is %d should be %d\n",
		 count + i, i, ibuf[i/4], (count + i) / 4);
	}
      count += transfer;
    }
    close(fd);

}
